@@ -116,15 +116,15 @@ extern void malloc_stats __P((void));
 struct mallinfo
 {
   int arena;    /* total space allocated from system */
-  int ordblks;  /* number of available chunks */
+  int ordblks;  /* number of non-inuse chunks */
   int smblks;   /* unused -- always zero */
-  int hblks;    /* unused -- always zero */
-  int hblkhd;   /* unused -- always zero */
+  int hblks;    /* number of mmapped regions */
+  int hblkhd;   /* total space in mmapped regions */
   int usmblks;  /* unused -- always zero */
   int fsmblks;  /* unused -- always zero */
   int uordblks; /* total allocated space */
-  int fordblks; /* available (unallocated) space */
-  int keepcost; /* unused -- always zero */
+  int fordblks; /* total non-inuse space */
+  int keepcost; /* top-most, releasable (via malloc_trim) space */
 };
 
 extern struct mallinfo mallinfo __P((void));
@@ -144,6 +144,13 @@ extern int mallopt __P((int __param, int __val));
 #define M_MMAP_MAX		-4
 #endif
 	 
+/* Record the state of all malloc variables in an opaque data structure. */
+extern void * malloc_get_state __P ((void));
+
+/* Restore the state of all malloc variables from data obtained with
+   malloc_get_state(). */
+extern int malloc_set_state __P ((void * __ptr));
+
 __END_DECLS
 
 
