@@ -21,9 +21,14 @@ static char sccsid[] = "@(#)gethostnamadr.c	6.39 (Berkeley) 1/4/90";
 
 #include "inetprivate.h"
 #include <syslog.h>
+#include <errno.h>
+#include <netdb.h>
+#include <sys/stat.h>
+#include <sys/ioctl.h>
+#include <fcntl.h>
 
 #if defined(USE_OPTIONS_H)
-# include "res_opts.h"
+# include "res_options.h"
 #endif
 
 #define	MAXALIASES	35
@@ -42,34 +47,34 @@ static char sccsid[] = "@(#)gethostnamadr.c	6.39 (Berkeley) 1/4/90";
  */
 #define STRICT
 
-#define SERVICE_NONE	0
-#define SERVICE_BIND	1
-#define SERVICE_HOSTS	2
-#define SERVICE_NIS	3
-#define SERVICE_MAX	3
+#define SERVICE_NONE  0
+#define SERVICE_BIND  1
+#define SERVICE_HOSTS 2
+#define SERVICE_NIS   3
+#define SERVICE_MAX   3
 
-#define CMD_ORDER	"order"
-#define CMD_TRIMDOMAIN	"trim"
-#define CMD_HMA		"multi"
-#define CMD_SPOOF	"nospoof"
-#define CMD_SPOOFALERT	"alert"
-#define CMD_REORDER	"reorder"
-#define CMD_ON		"on"
-#define CMD_OFF		"off"
-#define CMD_WARN	"warn"
-#define CMD_NOWARN	"warn off"
+#define CMD_ORDER       "order"
+#define CMD_TRIMDOMAIN  "trim"
+#define CMD_HMA         "multi"
+#define CMD_SPOOF       "nospoof"
+#define CMD_SPOOFALERT  "alert"
+#define CMD_REORDER     "reorder"
+#define CMD_ON          "on"
+#define CMD_OFF         "off"
+#define CMD_WARN        "warn"
+#define CMD_NOWARN      "warn off"
 
-#define ORD_BIND	"bind"
-#define ORD_HOSTS	"hosts"
-#define ORD_NIS		"nis"
+#define ORD_BIND        "bind"
+#define ORD_HOSTS       "hosts"
+#define ORD_NIS         "nis"
 
-#define ENV_HOSTCONF	"RESOLV_HOST_CONF"
-#define ENV_SERVORDER	"RESOLV_SERV_ORDER"
-#define ENV_SPOOF	"RESOLV_SPOOF_CHECK"
-#define ENV_TRIM_OVERR	"RESOLV_OVERRIDE_TRIM_DOMAINS"
-#define ENV_TRIM_ADD	"RESOLV_ADD_TRIM_DOMAINS"
-#define ENV_HMA		"RESOLV_MULTI"
-#define ENV_REORDER	"RESOLV_REORDER"
+#define ENV_HOSTCONF    "RESOLV_HOST_CONF"
+#define ENV_SERVORDER   "RESOLV_SERV_ORDER"
+#define ENV_SPOOF       "RESOLV_SPOOF_CHECK"
+#define ENV_TRIM_OVERR  "RESOLV_OVERRIDE_TRIM_DOMAINS"
+#define ENV_TRIM_ADD    "RESOLV_ADD_TRIM_DOMAINS"
+#define ENV_HMA         "RESOLV_MULTI"
+#define ENV_REORDER     "RESOLV_REORDER"
 
 #define TOKEN_SEPARATORS " ,;:"
 
@@ -98,7 +103,7 @@ static char *trimdomain[MAXTRIMDOMAINS];
 static char trimdomainbuf[BUFSIZ];
 static int numtrimdomains = 0;
 #ifdef NIS
-static struct hostent *_getnishost();
+static struct hostent *_getnishost __P((char *, char*));
 #endif
 
 #ifdef RESOLVSORT
@@ -121,20 +126,34 @@ typedef union {
     char ac;
 } align;
 
-#ifdef SHLIB
-extern int h_errno;
-#else
-extern int h_errno;
-#endif
-extern int errno;
-
 #if NLS
 #include "nl_types.h"
 #endif
+  
+extern char *__libc_secure_getenv(const char *);
+extern int yp_match   (char *, char *, char *, int, char **, int *);
+extern int yp_first   (char *, char *, char **, int *, char **, int *);
+extern int yp_next    (char *, char *, char *, int, char **,
+                       int *, char **, int *);
+extern int yp_get_default_domain   (char **);
+
+struct hostent *_gethtbyaddr(const char *, int, int );
+struct hostent *_gethtbyname( char * );
+struct hostent *_gethtent( void );
+void _endhtent( void );
+void _sethtent( int );
+struct hostent *gethostbyaddr(const char *, int, int );
+struct hostent *gethostbyname(const char *);
+struct hostent *gethostent( void );
+static struct hostent *getanswer( const querybuf *, int,
+											const char *, int, int );
+static void init_services( void );
+static void reorder_addrs( struct hostent * );
+static struct hostent *trim_domains( struct hostent * );
+static void dotrimdomain( char * );
 
 static void
-dotrimdomain(c)
-char *c;
+dotrimdomain( char *c )
 {
 	/* assume c points to the start of a host name; trim off any
 	   domain name matching any of the trimdomains */
@@ -149,8 +168,7 @@ char *c;
 }
 
 static struct hostent *
-trim_domains(h)
-struct hostent *h;
+trim_domains( struct hostent *h )
 {
 	if(numtrimdomains){
 		int i;
@@ -163,14 +181,14 @@ struct hostent *h;
 
 
 #if NLS
-#define bad_config_format(cmd) \
+#define bad_config_format(cmd,line) \
     fprintf(stderr, catgets(_libc_cat, NetMiscSet, \
 		NetMiscResolvIncorrectFormat, \
-		"resolv+: %s: \"%s\" command incorrectly formatted.\n"), \
-		hostconf, cmd);
+		"resolv+: %s:%d: command incorrectly formatted.\n"), \
+		hostconf, line);	/* for now, don't print cmd */
 #else
-#define bad_config_format(cmd) \
-    fprintf(stderr, "resolv+: %s: \"%s\" command incorrectly formatted.\n", hostconf, cmd);
+#define bad_config_format(cmd,line) \
+    fprintf(stderr, "resolv+: %s:%d: command incorrectly formatted.\n", hostconf, line);
 #endif
 
 /* static void debug_breakout(x)
@@ -211,10 +229,12 @@ long x;
 /* Hal Stern (June 1992) of Sun claimed that more than 4 ethernets in a
 Sun 4/690 would not work.  This variable is set to 10 to accomodate our
 version of reality */
-#define MAXINTERFACES (10)
+/* Alan Cox recommends now (Aug 1996) at minimum 64 since it's not unusual to
+have about 250 of them... */
+#define MAXINTERFACES (64)
 
-static void reorder_addrs(h)
-struct hostent *h;
+static void
+reorder_addrs( struct hostent *h )
 {
 	static struct {
 		char iname[16];
@@ -255,7 +275,9 @@ struct hostent *h;
 		endp = ifs.ifc_len / sizeof(struct ifreq);
 		itp = itab;
 		for (p = ifs.ifc_req; endp; p++, endp--) {
-			strcpy( itp->iname , p->ifr_name); /* copy interface name */
+			/* copy interface name */
+			strncpy(itp->iname, p->ifr_name, sizeof(itp->iname));
+			itp->iname[sizeof(itp->iname)-1] = 0;
 
 			err = ioctl(fd, SIOCGIFNETMASK, p); /* get netmask */
             if (err == -1) continue;	/* error? skip this interface */
@@ -300,26 +322,48 @@ struct hostent *h;
 }
 
 static void
-init_services()
+init_services( void )
 {
 	char *cp, *dp, buf[BUFSIZ];
 	register int cc = 0;
 	FILE *fd;
 	char *tdp = trimdomainbuf;
 	char *hostconf;
+	int myfd, lineno, errs;
 
 #if NLS
 	libc_nls_init();
 #endif
-	if(NULL==(hostconf=getenv(ENV_HOSTCONF))){
+	if(NULL==(hostconf=__libc_secure_getenv(ENV_HOSTCONF))){
 		hostconf=_PATH_HOSTCONF;
 	}
-	if ((fd = (FILE *)fopen(hostconf, "r")) == NULL) {
+
+	myfd = open(hostconf, O_RDONLY);
+	if (myfd >= 0) {
+		struct stat sb;
+		/* Sorry, only word readable files.
+		 * I wouldn't like it to be tricked by smart
+		 * "secure" setuid hackery. */
+		if (fstat(myfd, &sb) || !(sb.st_mode & S_IROTH)) {
+			close(myfd);
+			myfd = -1;
+		}
+	}
+	if (myfd < 0 || ((fd = (FILE *)fdopen(myfd, "r")) == NULL)) {
 				/* make some assumptions */
+		if (myfd >= 0)
+			close (myfd);
 		service_order[0] = SERVICE_BIND;
 		service_order[1] = SERVICE_NONE;
 	} else {
+		errs = lineno = 0;
 		while (fgets(buf, BUFSIZ, fd) != NULL) {
+			if (errs) {
+				/* FIXME: NLS */
+				fprintf(stderr, "resolv+: errors found, parsing aborted\n");
+				break;
+			}
+			lineno++;
 			if ((cp = rindex(buf, '\n')) != NULL)
 				*cp = '\0';
 			if (buf[0] == '#')
@@ -335,63 +379,65 @@ init_services()
 
 			for (cp=buf; cp && *cp && isspace(*cp); cp++);
 			if  (cp && *cp) {
-			strcpy(buf,cp);
-			if (checkbuf(buf, CMD_ORDER)) {
-				cp = strpbrk(buf, " \t");
-				if (!cp 
+				memmove(buf, cp, strlen(cp) + 1);
+				if (checkbuf(buf, CMD_ORDER)) {
+					cp = strpbrk(buf, " \t");
+					if (!cp 
 #ifdef STRICT
-				    || !*(cp+1)
+						 || !*(cp+1)
 #endif
-				    )
-                                {
-                                    bad_config_format(CMD_ORDER);
-                                } else
-                                {
-                                    do {
-					while (*cp == ' ' || *cp == '\t')
-						cp++;
-					dp = strpbrk(cp, TOKEN_SEPARATORS);
-					if (dp) *dp = '\0';
-					if (checkbuf(cp, ORD_BIND))
-						service_order[cc++] = SERVICE_BIND;
-					else if (checkbuf(cp, ORD_HOSTS))
-						service_order[cc++] = SERVICE_HOSTS;
-					else if (checkbuf(cp, ORD_NIS))
-						service_order[cc++] = SERVICE_NIS;
-					else
-                                        {
-                                            bad_config_format(CMD_ORDER);
+						 ) {
+						errs++; bad_config_format(CMD_ORDER, lineno);
+					} else {
+						do {
+							while (*cp == ' ' || *cp == '\t')
+									cp++;
+							dp = strpbrk(cp, TOKEN_SEPARATORS);
+							if (dp) *dp = '\0';
+							if (checkbuf(cp, ORD_BIND)) {
+								service_order[cc++] = SERVICE_BIND;
+								if (!(_res.options & RES_INIT))
+										res_init();
+							} else if (checkbuf(cp, ORD_HOSTS))
+									service_order[cc++] = SERVICE_HOSTS;
+							else if (checkbuf(cp, ORD_NIS))
+									service_order[cc++] = SERVICE_NIS;
+							else {
+								errs++; bad_config_format(CMD_ORDER, lineno);
 #if NLS
-			fprintf(stderr, catgets(_libc_cat, NetMiscSet,
-						NetMiscResolvInvalid,
-						"resolv+: \"%s\" is an invalid keyword\n"), cp);
-			fprintf(stderr, catgets(_libc_cat, NetMiscSet,
-						NetMiscResolvValid,
+								fprintf(stderr, catgets(_libc_cat, NetMiscSet,
+																NetMiscResolvInvalid,
+						"resolv+: %s:%d: invalid keyword\n"), hostconf, lineno);
+								fprintf(stderr, catgets(_libc_cat, NetMiscSet,
+																NetMiscResolvValid,
 						"resolv+: valid keywords are: %s, %s and %s\n"),
+										  ORD_BIND, ORD_HOSTS, ORD_NIS);
 #else
-                                            fprintf(stderr, "resolv+: \"%s\" is an invalid keyword\n", cp);
-                                            fprintf(stderr, "resolv+: valid keywords are: %s, %s and %s\n",
+								fprintf(stderr,
+						"resolv+: %s:%d is an invalid keyword\n", hostconf, lineno);
+								fprintf(stderr,
+										  "resolv+: valid keywords are: %s, %s and %s\n",
+										  ORD_BIND, ORD_HOSTS, ORD_NIS);
 #endif
-                                                   ORD_BIND, ORD_HOSTS, ORD_NIS);
 
-                                        }
+							}
 
-					if (dp) cp = ++dp;
-                                    } while (dp != NULL);
-                                    if (cc == 0)
-                                    {
-                                        bad_config_format(CMD_ORDER);
+							if (dp) cp = ++dp;
+						} while (dp != NULL);
+						if (cc == 0) {
+							errs++; bad_config_format(CMD_ORDER, lineno);
 #if NLS
-                                        fprintf(stderr,catgets(_libc_cat, NetMiscSet,
-							       NetMiscUnrecognized,
-							       "resolv+: search order not specified or unrecognized keyword, host resolution will fail.\n"));
+							fprintf(stderr,catgets(_libc_cat, NetMiscSet,
+														  NetMiscUnrecognized,
+"resolv+: search order not specified or unrecognized keyword, host resolution will fail.\n"));
 #else
-                                        fprintf(stderr,"resolv+: search order not specified or unrecognized keyword, host resolution will fail.\n");
+							fprintf(stderr,
+"resolv+: search order not specified or unrecognized keyword, host resolution will fail.\n");
 #endif
-                                    }
-                                }
+						}
+					}
 
-			} else if (checkbuf(buf, CMD_HMA)) {
+				} else if (checkbuf(buf, CMD_HMA)) {
 #ifdef STRICT
 
 #if NLS
@@ -403,101 +449,113 @@ init_services()
 				    checkbuf(cp,CMD_ON)) val = 1; \
 				else if (strlen(CMD_OFF) == strlen(cp) && \
 					 checkbuf(cp,CMD_OFF)) val = 0; \
-				else  { bad_config_format(key); \
+				else { \
+					errs++; bad_config_format(key, lineno); \
 					fprintf(stderr, catgets(_libc_cat, NetMiscSet, \
 						NetMiscResolvInvalid, \
-						"resolv+: \"%s\" is an invalid keyword\n"), cp); }\
-				} else bad_config_format(key); }
+						"resolv+: %s:%d: invalid keyword\n"), hostconf, lineno); \
+				     } \
+				} else { errs++; bad_config_format(key, lineno); } \
+			}
 #else
 
 #define check_legal(key,val) { \
 				for (cp=strpbrk(buf, " \t"); \
 				    cp && *cp && isspace(*cp); cp++); \
-				if (cp && *cp) {\
+				if (cp && *cp) { \
 				if (strlen(CMD_ON) == strlen(cp) && \
 				    checkbuf(cp,CMD_ON)) val = 1; \
 				else if (strlen(CMD_OFF) == strlen(cp) && \
 					 checkbuf(cp,CMD_OFF)) val = 0; \
-				else  { bad_config_format(key); \
-                                     fprintf(stderr, "resolv+: \"%s\" is an invalid keyword\n", cp); } \
-				} else bad_config_format(key); }
+				else  { \
+					errs++; bad_config_format(key, lineno); \
+               fprintf(stderr, "resolv+: %s:%d is an invalid keyword\n", hostconf, lineno); \
+				} \
+			} else { errs++; bad_config_format(key, lineno); } \
+		}
 #endif /* NLS */
-			    check_legal(CMD_HMA,hosts_multiple_addrs);
+					check_legal(CMD_HMA,hosts_multiple_addrs);
 
 #else /* !STRICT */
-                            if ( cp = strpbrk(buf, " \t") )
-                            {
-				while (*cp == ' ' || *cp == '\t') cp++;
-				if (checkbuf(cp, CMD_ON))
-					hosts_multiple_addrs = 1;
-                            } else
-                                bad_config_format(CMD_HMA);
+					if ( cp = strpbrk(buf, " \t") ) {
+						while (*cp == ' ' || *cp == '\t') cp++;
+						if (checkbuf(cp, CMD_ON))
+								hosts_multiple_addrs = 1;
+					} else {
+						errs++; bad_config_format(CMD_HMA, lineno);
+					}
 #endif /* STRICT */
 
-			} else if (checkbuf(buf, CMD_SPOOF)) {
+				} else if (checkbuf(buf, CMD_SPOOF)) {
 #ifdef STRICT
-			    check_legal(CMD_SPOOF,spoof);
+					check_legal(CMD_SPOOF,spoof);
 #else
-                            if ( cp = strpbrk(buf, " \t") )
-                            {
-				while (*cp == ' ' || *cp == '\t') cp++;
-				if (checkbuf(cp, CMD_ON))
-					spoof = 1;
-                            } else
-                                bad_config_format(CMD_SPOOF);
+					if ( cp = strpbrk(buf, " \t") ) {
+						while (*cp == ' ' || *cp == '\t') cp++;
+						if (checkbuf(cp, CMD_ON))
+								spoof = 1;
+					} else {
+						errs++; bad_config_format(CMD_SPOOF, lineno);
+					}
 #endif
 
 
-			} else if (checkbuf(buf, CMD_SPOOFALERT)) {
+				} else if (checkbuf(buf, CMD_SPOOFALERT)) {
 #ifdef STRICT
-			    check_legal(CMD_SPOOFALERT,spoofalert);
+					check_legal(CMD_SPOOFALERT,spoofalert);
 #else
-                            if ( cp = strpbrk(buf, " \t") )
-                            {
-				while (*cp == ' ' || *cp == '\t') cp++;
-				if (checkbuf(cp, CMD_ON))
-					spoofalert = 1;
-                            } else
-                                bad_config_format(CMD_SPOOFALERT);
+					if ( cp = strpbrk(buf, " \t") ) {
+						while (*cp == ' ' || *cp == '\t') cp++;
+						if (checkbuf(cp, CMD_ON))
+								spoofalert = 1;
+					} else {
+						errs++; bad_config_format(CMD_SPOOFALERT, lineno);
+					}
 #endif
 
-			} else if (checkbuf(buf, CMD_REORDER)) {
+				} else if (checkbuf(buf, CMD_REORDER)) {
 #ifdef STRICT
-			    check_legal(CMD_REORDER,reorder);
+					check_legal(CMD_REORDER,reorder);
 #else
-				if (cp = strpbrk(buf, " \t")) {
-				while (*cp == ' ' || *cp == '\t') cp++;
-				if (checkbuf(cp, CMD_ON))
-					reorder = 1;
-				} else
-					bad_config_format(CMD_REORDER);
+					if (cp = strpbrk(buf, " \t")) {
+						while (*cp == ' ' || *cp == '\t') cp++;
+						if (checkbuf(cp, CMD_ON))
+								reorder = 1;
+					} else {
+						errs++; bad_config_format(CMD_REORDER, lineno);
+					}
 #endif
 
-			} else if (checkbuf(buf, CMD_TRIMDOMAIN)) {
-				if(numtrimdomains<MAXTRIMDOMAINS){
-                                    if ( cp = strpbrk(buf, " \t") )
-                                    {
-					while (*cp == ' ' || *cp == '\t') cp++;
-                                        if (cp)
-                                        {
-                                            (void) strcpy(tdp,cp);
-                                            trimdomain[numtrimdomains++]=tdp;
-                                            tdp += strlen(cp)+1;
-                                        } else
-                                            bad_config_format(CMD_TRIMDOMAIN);
-                                    } else
-                                        bad_config_format(CMD_TRIMDOMAIN);
+				} else if (checkbuf(buf, CMD_TRIMDOMAIN)) {
+					if(numtrimdomains<MAXTRIMDOMAINS){
+						if (( cp = strpbrk(buf, " \t" ))) {
+							while (*cp == ' ' || *cp == '\t') cp++;
+							if (*cp) {
+								/* sec fix */
+								if (strlen(cp)+tdp-trimdomainbuf
+									 < sizeof(trimdomainbuf)-2) {
+									(void) strcpy(tdp,cp);
+									trimdomain[numtrimdomains++]=tdp;
+									tdp += strlen(cp)+1;
+								}
+							} else {
+								errs++; bad_config_format(CMD_TRIMDOMAIN, lineno);
+							}
+						} else {
+							errs++; bad_config_format(CMD_TRIMDOMAIN, lineno);
+						}
+					}
 				}
-			}
 #ifdef STRICT
 				else {
-					if (cp = strpbrk(buf, " \t")) *cp='\0';
+					if ((cp = strpbrk(buf, " \t"))) *cp='\0';
+					errs++;
 #if NLS
-			fprintf(stderr, catgets(_libc_cat, NetMiscSet,
-						NetMiscResolvInvalid,
-						"resolv+: \"%s\" is an invalid keyword\n"), buf);
+					fprintf(stderr, catgets(_libc_cat, NetMiscSet,
+													NetMiscResolvInvalid,
+						"resolv+: %s:%d: invalid keyword\n"), hostconf, lineno);
 #else
-					fprintf(stderr, "resolv+: \"%s\" is an invalid keyword\n", buf);
+					fprintf(stderr, "resolv+: %s:%d: invalid keyword\n", buf);
 #endif
 				}
 #endif
@@ -507,22 +565,24 @@ init_services()
 		fclose(fd);
 	}
 	/* override service_order if environment variable */
-	if(NULL!=(cp=getenv(ENV_SERVORDER))){
+	if(NULL!=(cp=__libc_secure_getenv(ENV_SERVORDER))) {
 		cc=0;
 		if(NULL!=(cp=strtok(cp, TOKEN_SEPARATORS))){
 			do{
-				if(checkbuf(cp, ORD_BIND))
+				if(checkbuf(cp, ORD_BIND)) {
 					service_order[cc++] = SERVICE_BIND;
-				else if (checkbuf(cp, ORD_HOSTS))
+					if (!(_res.options & RES_INIT))
+						res_init();
+				} else if (checkbuf(cp, ORD_HOSTS))
 					service_order[cc++] = SERVICE_HOSTS;
 				else if (checkbuf(cp, ORD_NIS))
 					service_order[cc++] = SERVICE_NIS;
-			} while(cp=strtok(NULL, TOKEN_SEPARATORS));
+			} while((cp=strtok(NULL, TOKEN_SEPARATORS)));
 		service_order[cc] = SERVICE_NONE;
 		}
 	}
 	/* override spoof if environment variable */
-	if(NULL!=(cp=getenv(ENV_SPOOF))){
+	if(NULL!=(cp=__libc_secure_getenv(ENV_SPOOF))) {
 		if(checkbuf(cp, CMD_WARN)){
 			spoof=1;
 			spoofalert=1;
@@ -538,58 +598,59 @@ init_services()
 	}
 
 	/* override hma if environment variable */
-	if(NULL!=(cp=getenv(ENV_HMA)))
+	if(NULL!=(cp=__libc_secure_getenv(ENV_HMA)))
 		if(checkbuf(cp, CMD_ON))
 			hosts_multiple_addrs=1;
 		else
 			hosts_multiple_addrs=0;
 
 	/* override reorder if environment variable */
-	if ((cp = getenv(ENV_REORDER)) != NULL)
+	if ((cp = __libc_secure_getenv(ENV_REORDER)) != NULL)
 		if (checkbuf(cp, CMD_ON))
 			reorder = 1;
 		else
 			reorder = 0;
 
 	/* add trimdomains from environment variable */
-	if(NULL!=(cp=getenv(ENV_TRIM_ADD))){
+	if(NULL!=(cp=__libc_secure_getenv(ENV_TRIM_ADD))) {
 		if(NULL!=(cp=strtok(cp, TOKEN_SEPARATORS))){
 			do{
 				if(numtrimdomains<MAXTRIMDOMAINS){
-					(void)strcpy(tdp, cp);
-					trimdomain[numtrimdomains++]=tdp;
-					tdp += strlen(cp)+1;
+					/* sec fix */
+					if (strlen(cp)+tdp-trimdomainbuf < sizeof(trimdomainbuf)-2) {
+						(void)strcpy(tdp, cp);
+						trimdomain[numtrimdomains++]=tdp;
+						tdp += strlen(cp)+1;
+					}
 				}
-			} while(cp=strtok(NULL, TOKEN_SEPARATORS));
+			} while((cp=strtok(NULL, TOKEN_SEPARATORS)));
 		}
 	}
 
 	/* override trimdomains from environment variable */
-	if(NULL!=(cp=getenv(ENV_TRIM_OVERR))){
+	if(NULL!=(cp=__libc_secure_getenv(ENV_TRIM_OVERR))) {
 		numtrimdomains=0;
 		tdp=trimdomainbuf;
 		if(NULL!=(cp=strtok(cp, TOKEN_SEPARATORS))){
 			do{
 				if(numtrimdomains<MAXTRIMDOMAINS){
-					(void)strcpy(tdp, cp);
-					trimdomain[numtrimdomains++]=tdp;
-					tdp += strlen(cp)+1;
+					/* sec fix */
+					if (strlen(cp)+tdp-trimdomainbuf < sizeof(trimdomainbuf)-2) {
+						(void)strcpy(tdp, cp);
+						trimdomain[numtrimdomains++]=tdp;
+						tdp += strlen(cp)+1;
+					}
 				}
-			} while(cp=strtok(NULL, TOKEN_SEPARATORS));
+			} while((cp=strtok(NULL, TOKEN_SEPARATORS)));
 		}
 	}
 
 	service_done = 1;
-	if (!(_res.options & RES_INIT))
-		res_init();
 }
 
 static struct hostent *
-getanswer(answer, anslen, qname, qclass, qtype)
-	const querybuf *answer;
-	int anslen;
-	const char *qname;
-	int qclass, qtype;
+getanswer( const querybuf *answer, int anslen, const char *qname,
+			 int qclass, int qtype )
 {
 	register const HEADER *hp;
 	register const u_char *cp;
@@ -600,7 +661,9 @@ getanswer(answer, anslen, qname, qclass, qtype)
 	int haveanswer, had_error;
 	int toobig = 0;
 	char tbuf[MAXDNAME+1];
+	const char *tname;
 
+	tname = qname;
 	host.h_name = NULL;
 	eom = answer->buf + anslen;
 	/*
@@ -696,12 +759,31 @@ getanswer(answer, anslen, qname, qclass, qtype)
 				had_error++;
 				continue;
 			}
-			strcpy(bp, tbuf);
+			strcpy(bp, tbuf);	/* cannot overflow */
 			host.h_name = bp;
 			bp += n;
 			buflen -= n;
 			continue;
 		}
+		if (qtype == T_PTR && type == T_CNAME) {
+			n = dn_expand(answer->buf, eom, cp, tbuf, sizeof tbuf);
+                        if (n < 0) {
+				had_error++;
+                                continue;
+                        }
+                        cp += n;
+                        /* Get canonical name. */
+                        n = strlen(tbuf) + 1;   /* for the \0 */
+                        if (n > buflen) {
+				had_error++;
+                                continue;
+                        }
+                        strcpy(bp, tbuf);	/* cannot overflow */
+                        tname = bp;
+                        bp += n;
+                        buflen -= n;
+                        continue;
+                }
 		if (type != qtype) {
 			syslog(LOG_NOTICE|LOG_AUTH,
 #if NLS
@@ -716,7 +798,7 @@ getanswer(answer, anslen, qname, qclass, qtype)
 		}
 		switch (type) {
 		case T_PTR:
-			if (strcasecmp(qname, bp) != 0) {
+			if (strcasecmp(tname, bp) != 0) {
 				syslog(LOG_NOTICE|LOG_AUTH,
 #if NLS
 	       catgets(_libc_cat, NetMiscSet, NetMiscAskedForGot, AskedForGot),
@@ -827,6 +909,7 @@ getanswer(answer, anslen, qname, qclass, qtype)
 #endif /*BSD*/
 		if (!host.h_name) {
 			n = strlen(qname) + 1;	/* for the \0 */
+			/* I'm not sure about this one. */
 			strcpy(bp, qname);
 			host.h_name = bp;
 		}
@@ -837,6 +920,54 @@ getanswer(answer, anslen, qname, qclass, qtype)
 	}
 }
 
+struct hostent *
+gethostent( void )
+{
+  int cc;
+  struct hostent *hp;
+
+  if (!service_done)
+    init_services();
+
+  for (cc = 0; service_order[cc] != SERVICE_NONE &&
+	cc <= SERVICE_MAX; cc++)
+  {
+    switch (service_order[cc])
+    {
+      case SERVICE_BIND:
+	break;
+
+      case SERVICE_HOSTS:
+	hp = _gethtent ();
+	if (h_addr_ptrs[1] && reorder)
+	  reorder_addrs(hp);
+	if (hp)
+	{
+	  h_errno = NETDB_SUCCESS;
+	  return hp;
+	}
+	h_errno = HOST_NOT_FOUND;
+	break;
+
+#ifdef NIS
+      case SERVICE_NIS:
+	hp = _getnishost(NULL, "hosts.byname");
+	if (h_addr_ptrs[1] && reorder)
+	  reorder_addrs(hp);
+	if (hp)
+	{
+	  h_errno = NETDB_SUCCESS;
+	  return hp;
+	}
+	h_errno = HOST_NOT_FOUND;
+	break;
+#endif /* NIS */
+
+    }
+  }
+  return ((struct hostent *) NULL);
+}
+
 struct hostent *
 gethostbyname(const char *name)
 {
@@ -845,7 +976,6 @@ gethostbyname(const char *name)
 	register int cc;
 	int n;
 	struct hostent *hp;
-	extern struct hostent *_gethtbyname();
 
 	/*
 	 * disallow names consisting only of digits/dots, unless
@@ -894,33 +1024,60 @@ gethostbyname(const char *name)
 		switch (service_order[cc]) {
 		case SERVICE_BIND:
 			if ((n = res_search(name, C_IN, T_A,
-					    buf.buf, sizeof(buf))) < 0) {
+					    buf.buf, sizeof(buf.buf))) < 0) {
 #ifdef DEBUG
 				if (_res.options & RES_DEBUG)
 					printf("res_search failed\n");
 #endif
+				break;
 			}
 			hp = getanswer(&buf, n, name, C_IN, T_A);
 			if (h_addr_ptrs[1] && reorder)
 				reorder_addrs(hp);
 			if (hp)
+			{
+				h_errno = NETDB_SUCCESS;
 				return trim_domains(hp);
+			}
 			break;
 		case SERVICE_HOSTS:
-			hp = _gethtbyname(name);
+			if (*trimdomain) {			
+				char *p;
+				p = strdup(name);
+				dotrimdomain(p);
+				hp = _gethtbyname(p);
+				free(p);
+			}
+			else
+			{
+				hp = _gethtbyname(name);
+			}
 			if (h_addr_ptrs[1] && reorder)
 				reorder_addrs(hp);
 			if (hp)
-				return hp;
+			{
+				h_errno = NETDB_SUCCESS;
+				return trim_domains(hp);
+			}
 			h_errno = HOST_NOT_FOUND;
 			break;
 #ifdef NIS
 		case SERVICE_NIS:
-			hp = _getnishost(name, "hosts.byname");
+			if (*trimdomain) {			
+				char *p;
+				p = strdup(name);
+				dotrimdomain(p);
+				hp = _getnishost(p, "hosts.byname");
+				free(p);
+			} else
+				hp = _getnishost(name, "hosts.byname");
 			if (h_addr_ptrs[1] && reorder)
 				reorder_addrs(hp);
 			if (hp)
-				return hp;
+			{
+				h_errno = NETDB_SUCCESS;
+				return trim_domains(hp);
+			}
 			h_errno = HOST_NOT_FOUND;
 			break;
 #endif /* NIS */
@@ -937,7 +1094,6 @@ gethostbyaddr(const char *addr, int len, int type)
 	register int cc;
 	register struct hostent *hp;
 	char qbuf[MAXDNAME];
-	extern struct hostent *_gethtbyaddr();
 
 	if (type != AF_INET) {
 		h_errno = NETDB_INTERNAL;
@@ -954,6 +1110,7 @@ gethostbyaddr(const char *addr, int len, int type)
 	while (service_order[cc] != SERVICE_NONE) {
 	        switch (service_order[cc]) {
 		case SERVICE_BIND:
+			/* will not overflow */
 			(void)sprintf(qbuf, "%u.%u.%u.%u.in-addr.arpa",
 				      ((unsigned)addr[3] & 0xff),
 				      ((unsigned)addr[2] & 0xff),
@@ -1014,11 +1171,17 @@ gethostbyaddr(const char *addr, int len, int type)
 					for (addrs = hp->h_addr_list;
 						*addrs; addrs++){
 						if (!bcmp(addrs[0], addr, len))
+						{
+							h_errno = NETDB_SUCCESS;
 							return trim_domains(hp);
+						}
 					}
 #else
 					if (!bcmp(hp->h_addr, addr, len)))
+					{
+						h_errno = NETDB_SUCCESS;
 						return trim_domains(hp);
+					}
 #endif
 					/* We've been spoofed */
 					h_errno = HOST_NOT_FOUND;
@@ -1049,6 +1212,7 @@ gethostbyaddr(const char *addr, int len, int type)
 #if BSD < 43 && !defined(h_addr)	/* new-style hostent structure */
 				hp->h_addr = h_addr_ptrs[0];
 #endif
+				h_errno = NETDB_SUCCESS;
 				return trim_domains(hp);
 			}
 			h_errno = HOST_NOT_FOUND;
@@ -1056,11 +1220,15 @@ gethostbyaddr(const char *addr, int len, int type)
 		case SERVICE_HOSTS:
 			hp = _gethtbyaddr(addr, len, type);
 			if (hp)
+			{
+				h_errno = NETDB_SUCCESS;
 				return hp;
+			}
 			h_errno = HOST_NOT_FOUND;
 			break;
 #ifdef NIS
 		case SERVICE_NIS:
+			/* won't overflow */
 			(void)sprintf(qbuf, "%u.%u.%u.%u",
 				      ((unsigned)addr[0] & 0xff),
 				      ((unsigned)addr[1] & 0xff),
@@ -1068,7 +1236,10 @@ gethostbyaddr(const char *addr, int len, int type)
 				      ((unsigned)addr[3] & 0xff));
 			hp = _getnishost(qbuf, "hosts.byaddr");
 			if (hp)
+			{
+				h_errno = NETDB_SUCCESS;
 				return hp;
+			}
 			h_errno = HOST_NOT_FOUND;
 			break;
 #endif /* NIS */
@@ -1079,8 +1250,7 @@ gethostbyaddr(const char *addr, int len, int type)
 }
 
 void
-_sethtent(f)
-	int f;
+_sethtent( int f )
 {
 	if (hostf == NULL)
 		hostf = fopen(HOSTDB, "r" );
@@ -1090,16 +1260,16 @@ _sethtent(f)
 }
 
 void
-_endhtent()
+_endhtent( void )
 {
-	if (hostf && !stayopen) {
+	if (hostf && !stayopen && !(_res.options & RES_STAYOPEN)) {
 		(void) fclose(hostf);
 		hostf = NULL;
 	}
 }
 
 struct hostent *
-_gethtent()
+_gethtent( void )
 {
 	char *p;
 	register char *cp, **q;
@@ -1163,6 +1333,9 @@ again:
  *  alias field contains that name, and the name field contains the primary
  *  name of the host. Unlike dns, however, this behavior will still take place
  *  even if the alias applies only to one of the interfaces.
+ *
+ *	I changed to return aliase list no matter what. H.J.
+ *
  *  - determining a "local" address to put first is dependant on the netmask
  *  being such that the least significant network bit is more significant
  *  than any host bit. Only strange netmasks will violate this.
@@ -1172,7 +1345,8 @@ again:
  *                     - John DiMarco <jdd@cdf.toronto.edu>
  */
 struct hostent *
-_gethtbyname(const char *name)
+_gethtbyname(char *name)
+/* _gethtbyname(const char *name) vch */
 {
 	register struct hostent *p;
 	register char **cp;
@@ -1188,6 +1362,8 @@ _gethtbyname(const char *name)
 	static struct hostent ht;
 	static char *aliases[MAXALIASES];
 	static char namebuf[MAXHOSTNAMELEN];
+	static char aliasebuf[BUFSIZ+1];
+	int gap = aliasebuf - hostbuf;
 
 	hap = ht_addr_ptrs;
 	lhap = loc_addr_ptrs;
@@ -1200,21 +1376,46 @@ _gethtbyname(const char *name)
 
 	aliases[0]=NULL;
 	aliases[1]=NULL;
-	(void) strcpy(namebuf, name);
+	(void) strncpy(namebuf, name, sizeof(namebuf));
+	namebuf[sizeof(namebuf)-1] = 0;
 
 	(void)gethostname(localname, sizeof(localname));
 
 	_sethtent(0);
-	while (p = _gethtent()) {
+	while ((p = _gethtent())) {
 		if (strcasecmp(p->h_name, name) == 0)
 			found++;
 		else
 			for (cp = p->h_aliases; *cp != 0; cp++)
 				if (strcasecmp(*cp, name) == 0){
 					found++;
+#if 0
 					aliases[0]=name;
-					(void) strcpy(namebuf, p->h_name);
+#endif
+					/*
+					 * This should not overflow, but it is
+					 * better to be safe than sorry.
+					 */
+					(void) strncpy(namebuf, p->h_name,
+						       sizeof(namebuf));
+					namebuf[sizeof(namebuf)-1] = 0;
 				}
+
+		cp = p->h_aliases;
+		if (found && aliases[0] == NULL && *cp)
+		{
+			int i;
+
+			memcpy (aliasebuf, hostbuf, sizeof(aliasebuf));
+			for (i = 0; *cp != 0 &&
+				i < sizeof (aliases) / sizeof (aliases [0]) - 1;
+				i++, cp++)
+			{
+				aliases [i] = *cp + gap;
+			}
+			aliases [i] = NULL;
+		}
+
 		if (strcasecmp(p->h_name, localname) == 0)
 			localfound++;
 		else
@@ -1314,23 +1515,45 @@ _gethtbyname(const char *name)
 
 #ifdef NIS
 static struct hostent *
-_getnishost(name, map)
-	char *name, *map;
+_getnishost( char *name, char *map )
 {
-	register char *cp, *dp, **q;
+	register char *cp, **q;
 	char *result;
 	int resultlen;
+	static int first = 1;
 	static struct hostent h;
 	static char *domain = (char *)NULL;
+	static char *keyname;
+	static int keylen;
 
 	if (domain == (char *)NULL)
 		if (yp_get_default_domain (&domain))
 			return ((struct hostent *)NULL);
 
-	if (yp_match(domain, map, name, strlen(name), &result, &resultlen))
+	if (name)
+	{
+	  if (yp_match(domain, map, name, strlen(name), &result, &resultlen))
 		return ((struct hostent *)NULL);
+	}
+	else
+	{
+	  /* If it is the first. */
+	  if (first)
+	  {
+	    if (yp_first(domain, map, &keyname, &keylen, &result,
+		&resultlen))
+	      return ((struct hostent *)NULL);
+	    first = 0;
+	  }
+	  else
+	  {
+	    if (yp_next (domain, map, keyname, keylen, &keyname,
+		&keylen, &result, &resultlen))
+	      return ((struct hostent *)NULL);
+	  }
+	}
 
-	if (cp = index(result, '\n'))
+	if ((cp = index(result, '\n')))
 		*cp = '\0';
 
 	cp = strpbrk(result, " \t");
@@ -1371,7 +1594,7 @@ _gethtbyaddr(const char *addr, int len, int type)
 	register struct hostent *p;
 
 	_sethtent(0);
-	while (p = _gethtent())
+	while (( p = _gethtent()))
 		if (p->h_addrtype == type && !bcmp(p->h_addr, addr, len))
 			break;
 	_endhtent();
@@ -1380,9 +1603,7 @@ _gethtbyaddr(const char *addr, int len, int type)
 
 #ifdef RESOLVSORT
 static void
-addrsort(ap, num)
-	char **ap;
-	int num;
+addrsort( char **ap, int num )
 {
 	int i, j;
 	char **p;
