@@ -1448,7 +1448,11 @@ group_number (char *w, char *rear_ptr, const char *grouping,
 
   /* We treat all negative values like CHAR_MAX.  */
 
+#ifdef __CHAR_UNSIGNED__
+  if (*grouping == CHAR_MAX )
+#else
   if (*grouping == CHAR_MAX || *grouping < 0)
+#endif
     /* No grouping should be done.  */
     return w;
 
@@ -1474,7 +1478,11 @@ group_number (char *w, char *rear_ptr, const char *grouping,
 	  if (*grouping == '\0')
 	    /* The previous grouping repeats ad infinitum.  */
 	    --grouping;
+#ifdef __CHAR_UNSIGNED__
+	  else if (*grouping == CHAR_MAX )
+#else
 	  else if (*grouping == CHAR_MAX || *grouping < 0)
+#endif
 	    {
 	      /* No further grouping to be done.
 		 Copy the rest of the number.  */
