@@ -285,6 +285,7 @@ xdr_ypresp_all(XDR *xdrs, struct ypresp_all *objp)
   return (TRUE);
 }
 
+#if 0
 bool_t
 xdr_ypmaplist_str (XDR *xdrs, char *objp)
 {
@@ -294,11 +295,16 @@ xdr_ypmaplist_str (XDR *xdrs, char *objp)
     }
   return (TRUE);
 }
+#endif
 
 bool_t
 xdr_ypmaplist(XDR *xdrs, struct ypmaplist *objp)
 {
+#if 0
   if (!xdr_ypmaplist_str(xdrs, objp->map))
+#else
+  if (!xdr_mapname(xdrs, &objp->map))
+#endif
     {
       return (FALSE);
     }
